@@ -3,53 +3,39 @@
 ##############################################################
 
 
-# setup generic function
-if (!isGeneric("slab"))
-  setGeneric("slab", function(data, fm, ...) standardGeneric("slab"))
+# SoilProfileCollection method
+slab.SPC <- function(data, fm, ...){
+	
+	# extract horizons and site 
+	h <- horizons(data)
+	s <- site(data)
+	
+	# if there is site data, join together
+	if(nrow(s) > 0)
+		h <- join(h, s, type='left', by=idname(data))
+	
+	# add old-style, hard-coded {id, top, bottom} column names        
+	h$id <- h[[idname(data)]]
+	hzDepthCols <- horizonDepths(data)
+	h$top <- h[[hzDepthCols[1]]]
+	h$bottom <- h[[hzDepthCols[2]]]
+	
+	# perform aggregation, using data.frame method
+	res <- slab(h, fm, ...)
+	
+	# result is a data.frame
+	return(res)
+}
 
-# temp interface to SPC class objects
-setMethod(f='slab', signature='SoilProfileCollection',
-  function(data, fm, ...){
-  
-  # extract horizons and site 
-  h <- horizons(data)
-  s <- site(data)
-  
-  # if there is site data, join together
-  if(nrow(s) > 0)
-    h <- join(h, s, type='left', by=idname(data))
-          
-  # add old-style, hard-coded {id, top, bottom} column names        
-  h$id <- h[[idname(data)]]
-  hzDepthCols <- horizonDepths(data)
-  h$top <- h[[hzDepthCols[1]]]
-  h$bottom <- h[[hzDepthCols[2]]]
-  
-  # perform aggregation, using data.frame method
-  res <- slab(h, fm, ...)
-  
-  ## TODO: there is no simple way to get back an SPC object, as there are several vars / slab returned
-  
-  # result is a data.frame
-  return(res)
-  }
-)
-
-
-##
-## TODO: update to current standards, see slice()
-##
-# current interface to data.frame objects
-setMethod(f='slab', signature='data.frame',
-definition=function(data, fm, progress='none', ...) {
-    
-    ## important: change the default behavior of data.frame and melt
-    opt.original <- options(stringsAsFactors = FALSE)
-    
+# data.frame method
+slab.DF <- function(data, fm, progress='none', ...) {
+	## important: change the default behavior of data.frame and melt
+	opt.original <- options(stringsAsFactors = FALSE)
+	
 	# sanity check:
 	if(! inherits(fm, "formula"))
 		stop('must provide a valid formula: groups ~ var1 + var2 + ...', call.=FALSE)
-	  
+	
 	# extract components of the formula:
 	g <- all.vars(update(fm, .~0)) # left-hand side
 	vars <- all.vars(update(fm, 0~.)) # right-hand side
@@ -65,12 +51,12 @@ definition=function(data, fm, progress='none', ...) {
 	if(any( !as.integer(data$top[data$top != 0]) == data$top[data$top != 0] ) | any( !as.integer(data$bottom) == data$bottom))
 		stop('This function can only accept integer horizon depths', call.=FALSE)
 	
-  # if there is no left-hand component in the formula, we are aggregating all data in the collection
-  if(g == '.') { 
-    g <- 'all.profiles' # add new grouping variable to horizons
-    data[, g] <- 1
-  }
-  
+	# if there is no left-hand component in the formula, we are aggregating all data in the collection
+	if(g == '.') { 
+		g <- 'all.profiles' # add new grouping variable to horizons
+		data[, g] <- 1
+	}
+	
 	# convert into long format
 	d.long <- melt(data, id.vars=c('id','top','bottom', g), measure.vars=vars)
 	
@@ -81,25 +67,42 @@ definition=function(data, fm, progress='none', ...) {
 	# apply slotting group-wise and return in long format
 	# note '...' is gobbled by soil.slot()
 	d.slotted <- ddply(d.long, .variables=c('variable', g), .progress=progress, .parallel=getOption('AQP_parallel', default=FALSE), .fun=soil.slot, ...) 
-		
+	
 	# convert tops and bottoms to integers
 	d.slotted$top <- as.integer(d.slotted$top)
 	d.slotted$bottom <- as.integer(d.slotted$bottom)
 	
 	# reset options:
-    options(opt.original)
+	options(opt.original)
 	
 	# done
 	return(d.slotted)
-	}	
-)
+}
+
+
+# setup generic function
+if (!isGeneric("slab"))
+	setGeneric("slab", function(data, fm, ...) standardGeneric("slab"))
+
+
+##
+## TODO: update to current standards, see slice()
+##
+# current interface to data.frame objects
+setMethod(f='slab', signature='data.frame', definition=slab.DF)
+
+## TODO: integrate methods for SPC / data.frame
+## TODO: there is no simple way to get back an SPC object, as there are several vars / slab returned
+## TODO: use.wts doesn't work with SPC objects
+# temp interface to SPC class objects
+setMethod(f='slab', signature='SoilProfileCollection', definition=slab.SPC)
+
 
 ## 
 ## calculation of segment-wise summary statistics
 ## 
 ##TODO: figure out how to do weighted tables
-seg.summary <- function(l.recon, prop.class, use.wts, user.fun, l.recon.wts=NA, prop.levels=NA, class_prob_mode=1)
-	{
+seg.summary <- function(l.recon, prop.class, use.wts, user.fun, l.recon.wts=NA, prop.levels=NA, class_prob_mode=1) {
 	
 	# this is the number of slices contributing the the slice-wise aggregate
 	contributing_fraction <- sapply(l.recon, function(i) length(na.omit(i)) / length(i))
@@ -124,8 +127,7 @@ seg.summary <- function(l.recon, prop.class, use.wts, user.fun, l.recon.wts=NA,
 		
 		
 		# profile-weighted statistics
-		if(use.wts == TRUE)
-			{			
+		if(use.wts == TRUE) {
 			# weighted mean calculation: reduces to standard mean, when weights are equal
 			p.wtmean <- sapply(l.seq, function(i) wtd.mean(l.recon[[i]], weights=l.recon.wts[[i]], na.rm=TRUE)) 
 			
@@ -140,8 +142,7 @@ seg.summary <- function(l.recon, prop.class, use.wts, user.fun, l.recon.wts=NA,
 		
 		# try user defined function
 		# TODO: catch errors
-		if(!is.null(user.fun))
-			{
+		if(!is.null(user.fun)) {
 			p.user <- try( sapply(l.recon, user.fun) )
 			
 			if(use.wts)
@@ -150,8 +151,7 @@ seg.summary <- function(l.recon, prop.class, use.wts, user.fun, l.recon.wts=NA,
 				df.stats <- data.frame(contributing_fraction, p.mean, p.sd, p.quantiles, p.user)
 			}
 		# no user function	
-		else
-			{
+		else {
 			if(use.wts)
 				df.stats <- data.frame(contributing_fraction, p.mean, p.wtmean, p.sd, p.wtsd, p.quantiles, p.wtquantiles)
 			else
@@ -165,28 +165,24 @@ seg.summary <- function(l.recon, prop.class, use.wts, user.fun, l.recon.wts=NA,
 	## xtabs() is another approach, but it drops the count of NA
 	## xtabs(wts ~ obs, data=data.frame(wts=wts, obs=tf))
 	# categorical variables
-	if(prop.class == 'factor')
-		{
+	if(prop.class == 'factor') {
 		# get a vector of all possible categories
 		# note that l.recon contains factor codes
 		p.unique.classes <- as.vector(na.omit(unique(unlist(l.recon))))
 		
 		# tabular frequences for complete set of possible categories
-		p.table <- sapply(l.seq, function(i, cpm=class_prob_mode, wts=l.recon.wts) 
-			{
+		p.table <- sapply(l.seq, function(i, cpm=class_prob_mode, wts=l.recon.wts) {
 			tf <- factor(l.recon[[i]], levels=p.unique.classes, labels=prop.levels[p.unique.classes])
 			
 			# probabilities are relative to number of contributing profiles
-			if(cpm == 1)
-			  {
+			if(cpm == 1) {
 			  tb <- table(tf, useNA='no')
 			  # convert to proportions
 			  pt <- prop.table(tb)
 			  }
 			
 			# probabilities are relative to total number of profiles
-			else if(cpm == 2)
-			  {
+			else if(cpm == 2) {
 			  tb <- table(tf, useNA='always')
 			  # convert to proportions, 
 			  # the last column will be named 'NA', and contains the tally of NAs --> remove it
@@ -206,8 +202,7 @@ seg.summary <- function(l.recon, prop.class, use.wts, user.fun, l.recon.wts=NA,
 		# when there is only 1 class, things are more complicated:
 		# 1. no need to transpose p.table
 		# 2. need to manually add the class name
-		else
-			{
+		else {
 			p.prop <- data.frame(p.table)
 			names(p.prop) <- prop.levels[p.unique.classes]
 			}
@@ -229,8 +224,7 @@ seg.summary <- function(l.recon, prop.class, use.wts, user.fun, l.recon.wts=NA,
 # TODO: we only need x.recon for 1cm aggregation, otherwise l.recon is used
 # TODO: return the number of profiles + number of unique horizons when using custom segmenting interval
 # TODO: replace by() with equivilant plyr functions
-soil.slot <- function(data, seg_size=NA, seg_vect=NA, use.wts=FALSE, strict=FALSE, user.fun=NULL, class_prob_mode=1)
-	{
+soil.slot <- function(data, seg_size=NA, seg_vect=NA, use.wts=FALSE, strict=FALSE, user.fun=NULL, class_prob_mode=1) {
 	
 	#################################################################################
 	##### Initialization: check for fatal errors, and do some clean-up
@@ -244,14 +238,12 @@ soil.slot <- function(data, seg_size=NA, seg_vect=NA, use.wts=FALSE, strict=FALS
 	hz.test.top <- is.na(data$top)
 	hz.test.bottom <- is.na(data$bottom)
 	
-	if(any(hz.test.top))
-		{
+	if(any(hz.test.top)) {
 		print(data[which(hz.test.top), ])
 		stop('NA in horizon top boundary', call.=FALSE)
 		}
 		
-	if(any(hz.test.bottom))
-		{
+	if(any(hz.test.bottom)) {
 		print(data[which(hz.test.bottom), ])
 		stop('NA in horizon bottom boundary', call.=FALSE)
 		}
@@ -268,14 +260,12 @@ soil.slot <- function(data, seg_size=NA, seg_vect=NA, use.wts=FALSE, strict=FALS
 	prop.class <- class(data$prop)
 	
 	# if we have a character, then convert to factor
-	if(prop.class == 'character')
-		{
+	if(prop.class == 'character') {
 		message('Note: converting categorical variable to factor.')
 		data$prop <- factor(data$prop)
 		}
 	
-	if(prop.class == 'factor')
-		{
+	if(prop.class == 'factor') {
 		# save the levels of our categorical variable
 		prop.levels <- levels(data$prop) 
 		}
@@ -289,20 +279,18 @@ soil.slot <- function(data, seg_size=NA, seg_vect=NA, use.wts=FALSE, strict=FALS
 	
 
 	#################################################################################
-	##### Step 1: unroll profiles in the collection: result is a list
+	##### Step 1: unroll profiles in the collection: result is a list    ############
+  ##### TODO: this could be done with slice()                          ############
 	#################################################################################
-	x.unrolled <- dlply(data, .(id), .fun=function(i, m=max_d) 
-		{
+	x.unrolled <- dlply(data, "id", .fun=function(i, m=max_d) {
 		
 		u <- try(unroll(top=i$top, bottom=i$bottom, prop=i$prop, max_depth=m, strict=strict))
 		
 		## TODO: could be better
 		# check for a non-NULL attribute of 'class'
 		# this will only happen when there was an error
-		if( !is.null(attr(u, 'class')))
-			{
-			if(attr(u, 'class') == 'try-error')
-				{
+		if( !is.null(attr(u, 'class'))) {
+			if(attr(u, 'class') == 'try-error') {
 				print(i)
 				stop('Error: bad horizon structure')
 				}
@@ -313,15 +301,16 @@ soil.slot <- function(data, seg_size=NA, seg_vect=NA, use.wts=FALSE, strict=FALS
 	
 	
 	#######################################################################################
-	##### Step 2: reconstitute into a matrix with 1:n-segment rows, and n_pedons columns ##
+	##### Step 2: reconstitute into a matrix with 1:n-segment rows, and n-pedons columns ##
 	##### TODO: use lists for everything, so that segment size does not affect agg. calcs #
+	##### TODO: don't use hard-coded weight column
 	#######################################################################################
 	# values
 	x.recon <- sapply(x.unrolled, '[')
 	
+  ## TODO: weights should be stored as a 'site' atttribute in an SPC object
 	# weights
-	if(use.wts == TRUE)
-		{
+	if(use.wts == TRUE) {
 		message('Note: profile weights are still experimental, use with caution!')
 		
 		# unroll a weight vector for each pedon
@@ -336,7 +325,7 @@ soil.slot <- function(data, seg_size=NA, seg_vect=NA, use.wts=FALSE, strict=FALS
 		}
 
 
-	## TODO: this is a little wasteful, as we can work with the x.unrolled instead
+	## TODO: this is wasteful, as we can work with the x.unrolled instead
 	# convert to lists
 	l.recon <- by(x.recon, 1:max_d, unlist)
 		
@@ -353,8 +342,7 @@ soil.slot <- function(data, seg_size=NA, seg_vect=NA, use.wts=FALSE, strict=FALS
 	# if we have a regular-interval segment size, re-group the data following the segmenting id
 	# note that we are testing for the default value of seg_size and seg_vect
 	# must be a better way to do this..
-	if(!missing(seg_size) | !missing(seg_vect))
-		{
+	if(!missing(seg_size) | !missing(seg_vect)) {
 				
 		# use a user-defined segmenting vector
 		if(!missing(seg_vect))
@@ -386,8 +374,7 @@ soil.slot <- function(data, seg_size=NA, seg_vect=NA, use.wts=FALSE, strict=FALS
 			}
 			
 		# using a fixed-interval segmenting vector
-		else
-			{
+		else {
 			# generate a vector of unique segment ids
 			segment_label <- 1:((max_d/seg_size)+1)
 			
@@ -410,8 +397,7 @@ soil.slot <- function(data, seg_size=NA, seg_vect=NA, use.wts=FALSE, strict=FALS
 		
 		
 		# user-defined segmenting vector, starting from 0
-		if(!missing(seg_vect))
-			{
+		if(!missing(seg_vect)) {
 			
 			# if the user requests multiple slices that are beyond the length of the deepest profile
 			# throw an error
@@ -424,8 +410,7 @@ soil.slot <- function(data, seg_size=NA, seg_vect=NA, use.wts=FALSE, strict=FALS
 			
 			# the actual max depth may be less than the requested segments
 			# in that case we will need to truncate the horizon label vector to max_d
-			if(max_d < max.seg_vect)
-				{
+			if(max_d < max.seg_vect) {
 				seg_vect_legal_idx <- which( (seg_vect - max_d) <= 0)
 				sv_clean <- c(seg_vect[seg_vect_legal_idx], max_d)
 				warning(paste('Note: truncating requested lower segment (', max.seg_vect, ') to max profile depth (', max_d, ').', sep=''), call.=FALSE)
@@ -433,8 +418,7 @@ soil.slot <- function(data, seg_size=NA, seg_vect=NA, use.wts=FALSE, strict=FALS
 				
 			# the actual depth may be more than, or equal to, the deepest requested segment
 			# in that case we are truncating aggregation to the deepest requested lower boundary
-			else
-				{
+			else {
 				sv_clean <- seg_vect
 				}
 		
@@ -447,8 +431,7 @@ soil.slot <- function(data, seg_size=NA, seg_vect=NA, use.wts=FALSE, strict=FALS
 			# check for lower horizon where top == bottom, and remove it
 			bad_hz_list_TF <- with(df.top_bottom, top == bottom)
 			
-			if(TRUE %in% bad_hz_list_TF)
-				{
+			if(TRUE %in% bad_hz_list_TF) {
 				bad_hz_list_idx <- which(bad_hz_list_TF)
 				warning(paste('removing horizon with 0 thickness (hz ', bad_hz_list_idx, ')', sep=''), call.=FALSE)
 				df.top_bottom <- df.top_bottom[-bad_hz_list_idx, ]
@@ -457,8 +440,7 @@ soil.slot <- function(data, seg_size=NA, seg_vect=NA, use.wts=FALSE, strict=FALS
 			} # done with user-defined segment vector
 			
 		# using a fixed-interval segmenting vector
-		else
-			{
+		else {
 			# get the length of our segmented data set
 			# and generate a new sequence of depths
 			len <- length(l.recon)
@@ -473,22 +455,19 @@ soil.slot <- function(data, seg_size=NA, seg_vect=NA, use.wts=FALSE, strict=FALS
 		
 		# compute segment-wise summary statistics
 		df.stats <- seg.summary(l.recon, prop.class, use.wts, user.fun, l.recon.wts, prop.levels, class_prob_mode)
-		
 		} # done with segmenting
 	
 	
 	###############################################################
 	##### Step 3b: compute stats along single-interval segments  ##
 	###############################################################
-	else
-		{
+	else {
 	
 		# make the top and bottom hz labels
 		df.top_bottom <- data.frame(top=0:(max_d-1), bottom=1:max_d)
 		
 		# compute row-wise summary statistics
 		df.stats <- seg.summary(l.recon, prop.class, use.wts, user.fun, l.recon.wts, prop.levels, class_prob_mode)
-		
 		} # done with 1-unit interval aggregation
 
 	
@@ -496,13 +475,11 @@ soil.slot <- function(data, seg_size=NA, seg_vect=NA, use.wts=FALSE, strict=FALS
 	
 	## form into dataframe for returning to the user 
 	# this is usually where we have problems, caused by bad horizon boundaries
-	if(nrow(df.top_bottom) == nrow(df.stats))
-		{
+	if(nrow(df.top_bottom) == nrow(df.stats)) {
 		x.slotted <- data.frame(df.top_bottom, df.stats)
 		}
 	# something is wrong
-	else
-		{
+	else {
 		stop('The number of rows in aggregate data do not match number of segments. This was probably caused by incorrect horizon boundaries.')
 		}
 	
